"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mime_normalizer_1 = require("@vereign/mime-normalizer");
const dom_1 = require("@vereign/dom");
const MailParser_1 = require("../MailParser/MailParser");
const CryptoService_1 = require("../CryptoService");
const index_1 = require("./index");
const rka_1 = require("../../utils/rka");
const common_1 = require("../../utils/common");
const VerificationError_1 = require("../VerificationService/VerificationError");
/**
 * @deprecated
 */
const prepareMimeForPartsVerification = (mimeString, attachmentsHashes, senderSystem, sourceHtmlRabinFingerprint, sourceHtmlSize, sourcePlainRabinFingerprint, sourcePlainSize) => __awaiter(void 0, void 0, void 0, function* () {
    const { htmlPart, plainPart } = getNormalizedMimeParts(mimeString, senderSystem);
    const originalHtml = findSubstring(htmlPart, sourceHtmlRabinFingerprint, sourceHtmlSize);
    if (originalHtml === null) {
        throw new VerificationError_1.default("Unable to verify MIME. Source HTML part was not found.");
    }
    const originalPlain = findSubstring(plainPart, sourcePlainRabinFingerprint, sourcePlainSize);
    if (originalPlain === null) {
        throw new VerificationError_1.default("Unable to verify MIME. Source Plain part was not found.");
    }
    return yield index_1.default.preparePartsForSigning([
        originalHtml,
        originalPlain,
    ]);
});
const getNormalizedMimeParts = (mimeString, senderSystem) => {
    const { htmlPart, plainPart, } = index_1.default.getMimeHtmlAndPlainParts(mimeString);
    const normalizedHtmlPart = normalizeVendorHtml(htmlPart, senderSystem);
    const normalizedPlainPart = normalizePlainPart(plainPart);
    return {
        htmlPart: normalizedHtmlPart,
        plainPart: normalizedPlainPart,
    };
};
const findSubstring = (string, rabinFingerprint, substringSize) => {
    if (!string.length) {
        return "";
    }
    const startingIndex = rka_1.default.searchFingerprintInText(string, rabinFingerprint, substringSize);
    if (startingIndex === -1) {
        return null;
    }
    return string.slice(startingIndex, startingIndex + substringSize);
};
const getMimeHtmlAndPlainParts = (mimeString) => {
    mimeString = MailParser_1.fixNewLines(mimeString);
    const mimeParts = MailParser_1.default.parseMIME(mimeString);
    // TODO: check, whether we need to support 2+ html/plain MIME parts.
    const htmlPart = MailParser_1.default.getHTML(mimeString, mimeParts);
    const plainPart = MailParser_1.default.getPlain(mimeString, mimeParts);
    return {
        htmlPart,
        plainPart,
    };
};
/**
 * Function implementing specification
 * https://code.vereign.com/light/documentation/-/blob/master/Validation.md#partssignaturealgorithm
 *
 * @param parts - array of strings representing parts
 * @param options - hashing options
 */
const preparePartsForSigning = (parts, options = {
    hashParts: true,
}) => __awaiter(void 0, void 0, void 0, function* () {
    if (options.hashParts) {
        let hashFunction = options.hashFunction;
        if (!hashFunction) {
            hashFunction = (text) => __awaiter(void 0, void 0, void 0, function* () {
                return common_1.arrayBufferToBase64(yield CryptoService_1.default.SHA256(text));
            });
        }
        parts = yield Promise.all(parts.map((part) => __awaiter(void 0, void 0, void 0, function* () {
            return yield hashFunction(part);
        })));
    }
    return parts.sort().join("\n");
});
const normalizeVendorHtml = (htmlString, vendor) => {
    let document;
    if (typeof DOMParser !== "undefined") {
        const parser = new DOMParser();
        document = parser.parseFromString(htmlString, "text/html");
    }
    else {
        const { window } = new dom_1.DOM(htmlString);
        document = window.document;
    }
    return mime_normalizer_1.HTMLNormalizer.normalizeVendorHtml(document, vendor);
};
const normalizePlainPart = mime_normalizer_1.PlainNormalizer.normalizePlain;
exports.default = {
    normalizeVendorHtml,
    getMimeHtmlAndPlainParts,
    preparePartsForSigning,
    prepareMimeForPartsVerification,
    getNormalizedMimeParts,
    normalizePlainPart,
};
