"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findSeverityStateIndex = exports.SEVERITY_MESSAGE_PER_STATE = exports.SEVERITY_LEVEL_PER_STATE = exports.SEVERITY_STATES = exports.SEVERITY_LEVELS = void 0;
exports.SEVERITY_LEVELS = {
    VALID: 0,
    WARNING: 1,
    FAILURE: 2,
};
/**
 * Representation of
 * https://code.vereign.com/light/documentation/-/blob/master/Validation.md#verification-table
 *
 * Values meanings:
 * 0 - false
 * 1 - true
 * -1 - false/true
 *
 * By indices:
 * 0 - html valid
 * 1 - plain valid
 * 2 - pseudoPlain valid
 * 3 - attachments valid
 */
exports.SEVERITY_STATES = [
    [1, 1, 1, 1],
    [0, 1, 1, 1],
    [1, 0, 1, 1],
    [-1, -1, 0, 1],
    [0, 0, 1, 1],
    [-1, -1, -1, 0],
];
exports.SEVERITY_LEVEL_PER_STATE = [
    exports.SEVERITY_LEVELS.VALID,
    exports.SEVERITY_LEVELS.WARNING,
    exports.SEVERITY_LEVELS.WARNING,
    exports.SEVERITY_LEVELS.FAILURE,
    exports.SEVERITY_LEVELS.FAILURE,
    exports.SEVERITY_LEVELS.FAILURE,
];
exports.SEVERITY_MESSAGE_PER_STATE = [
    "Message is valid",
    "Message seems authentic, but HTML layout does not verify: Switch of formatting. Caution advised.",
    "The plain text of this message does not verify. Caution advised.",
    "CAUTION! This message has been tampered with. Do not click on any links, do not rely on its information. Contact the sender.",
    "CAUTION! This message has been tampered with. Do not click on any links, do not rely on its information. Contact the sender.",
    "The message is authentic, but at least one attachment appears to have been altered. Caution advised for the attachments, do not open!",
];
exports.findSeverityStateIndex = (htmlValid, plainValid, pseudoPlainValid, attachmentsValid) => {
    let index = -1;
    const verificationState = [
        +htmlValid,
        +plainValid,
        +pseudoPlainValid,
        +attachmentsValid,
    ];
    for (let i = 0; i < exports.SEVERITY_STATES.length; i++) {
        const severitySate = exports.SEVERITY_STATES[i];
        const match = severitySate.every((entry, index) => {
            if (entry === -1) {
                return true;
            }
            else {
                return verificationState[index] === entry;
            }
        });
        if (match) {
            index = i;
            break;
        }
    }
    return index;
};
