"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.escapeHtmlString = exports.arrayBufferToHex = exports.decompressData = exports.compressData = exports.ensureUint8Array = exports.ensureArrayBuffer = exports.ensureBase64 = exports.base64ToArrayBuffer = exports.arrayBufferToBase64 = void 0;
const zlib_min_1 = require("zlibjs/bin/zlib.min");
exports.arrayBufferToBase64 = (buffer) => {
    return Buffer.from(buffer).toString("base64");
};
exports.base64ToArrayBuffer = (base64) => {
    return Buffer.from(base64, "base64");
};
exports.ensureBase64 = (data) => {
    return typeof data === "string" ? data : exports.arrayBufferToBase64(data);
};
exports.ensureArrayBuffer = (data) => {
    return typeof data === "string" ? exports.base64ToArrayBuffer(data) : data;
};
exports.ensureUint8Array = (data) => {
    return data instanceof Uint8Array
        ? data
        : typeof data === "string"
            ? new Uint8Array(exports.base64ToArrayBuffer(data))
            : data instanceof ArrayBuffer
                ? new Uint8Array(data)
                : data;
};
exports.compressData = (binary) => {
    const deflate = new zlib_min_1.Zlib.Deflate(exports.ensureUint8Array(binary));
    return deflate.compress();
};
exports.decompressData = (binary) => {
    const inflate = new zlib_min_1.Zlib.Inflate(exports.ensureUint8Array(binary));
    return inflate.decompress();
};
exports.arrayBufferToHex = (buffer) => {
    return [...new Uint8Array(buffer)]
        .map((b) => b.toString(16).padStart(2, "0"))
        .join("");
};
exports.escapeHtmlString = (string) => {
    const matchHtmlRegExp = /["'&<>]/;
    const str = "" + string;
    const match = matchHtmlRegExp.exec(str);
    if (!match) {
        return str;
    }
    let escape;
    let html = "";
    let index = 0;
    let lastIndex = 0;
    for (index = match.index; index < str.length; index++) {
        switch (str.charCodeAt(index)) {
            case 34: // "
                escape = "&quot;";
                break;
            case 38: // &
                escape = "&amp;";
                break;
            case 39: // '
                escape = "&#39;";
                break;
            case 60: // <
                escape = "&lt;";
                break;
            case 62: // >
                escape = "&gt;";
                break;
            default:
                continue;
        }
        if (lastIndex !== index) {
            html += str.substring(lastIndex, index);
        }
        lastIndex = index + 1;
        html += escape;
    }
    return lastIndex !== index ? html + str.substring(lastIndex, index) : html;
};
exports.default = {
    compressData: exports.compressData,
    decompressData: exports.decompressData,
    arrayBufferToHex: exports.arrayBufferToHex,
    arrayBufferToBase64: exports.arrayBufferToBase64,
    base64ToArrayBuffer: exports.base64ToArrayBuffer,
    ensureUint8Array: exports.ensureUint8Array,
    ensureArrayBuffer: exports.ensureArrayBuffer,
    ensureBase64: exports.ensureBase64,
    escapeHtmlString: exports.escapeHtmlString,
};
