"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const findChromakeyBoundaries = (width, height, imageData, chromakeyWidth, chromakeyHeight) => {
    const maskSizeX = chromakeyWidth;
    const maskSizeY = chromakeyHeight;
    const totalSum = maskSizeX * maskSizeY * 255;
    const matrixR = [];
    const matrixG = [];
    const matrixB = [];
    for (let i = 0; i < width; i++) {
        matrixR[i] = [];
        matrixG[i] = [];
        matrixB[i] = [];
        for (let j = 0; j < height; j++) {
            matrixR[i][j] = 0;
            matrixG[i][j] = 0;
            matrixB[i][j] = 0;
        }
    }
    const maxDistance = Math.sqrt(3 * totalSum * totalSum);
    const suitableSimilarity = 0.6;
    let bestSimilarity = 0;
    let bestX = -1;
    let bestY = -1;
    for (let x = 0; x < width; x++) {
        for (let y = 0; y < height; y++) {
            const index = (y * width + x) * 4;
            const r = imageData.data[index];
            const g = imageData.data[index + 1];
            const b = imageData.data[index + 2];
            matrixR[x][y] += r;
            matrixG[x][y] += g;
            matrixB[x][y] += b;
            if (x - 1 >= 0) {
                matrixR[x][y] += matrixR[x - 1][y];
                matrixG[x][y] += matrixG[x - 1][y];
                matrixB[x][y] += matrixB[x - 1][y];
            }
            if (y - 1 >= 0) {
                matrixR[x][y] += matrixR[x][y - 1];
                matrixG[x][y] += matrixG[x][y - 1];
                matrixB[x][y] += matrixB[x][y - 1];
            }
            if (x - 1 >= 0 && y - 1 >= 0) {
                matrixR[x][y] -= matrixR[x - 1][y - 1];
                matrixG[x][y] -= matrixG[x - 1][y - 1];
                matrixB[x][y] -= matrixB[x - 1][y - 1];
            }
            if (x >= maskSizeX && y >= maskSizeY) {
                const tempSumR = matrixR[x][y] +
                    matrixR[x - maskSizeX][y - maskSizeY] -
                    (matrixR[x - maskSizeX][y] + matrixR[x][y - maskSizeY]);
                const tempSumG = matrixG[x][y] +
                    matrixG[x - maskSizeX][y - maskSizeY] -
                    (matrixG[x - maskSizeX][y] + matrixG[x][y - maskSizeY]);
                const tempSumB = matrixB[x][y] +
                    matrixB[x - maskSizeX][y - maskSizeY] -
                    (matrixB[x - maskSizeX][y] + matrixB[x][y - maskSizeY]);
                const distance = Math.sqrt(Math.pow(0 - tempSumR, 2) +
                    Math.pow(totalSum - tempSumG, 2) +
                    Math.pow(0 - tempSumB, 2));
                const similarity = 1 - distance / maxDistance;
                if (similarity > bestSimilarity) {
                    bestX = x;
                    bestY = y;
                    bestSimilarity = similarity;
                }
            }
        }
    }
    if (bestX >= 0 && bestY >= 0 && bestSimilarity > suitableSimilarity) {
        return {
            fromL: bestX - maskSizeX,
            fromT: bestY - maskSizeY,
            toL: bestX - maskSizeX + chromakeyWidth,
            toT: bestY - maskSizeY + chromakeyHeight,
        };
    }
    return { fromL: -1, fromT: -1, toL: -1, toT: -1 };
};
const putQrCodeOnChromakeyTemplate = (qrCodeImageBase64, templateImageBase64, placeholderWidth, placeholderHeight, scale = 1) => __awaiter(void 0, void 0, void 0, function* () {
    const qrCodeImage = yield loadImage(qrCodeImageBase64);
    const templateImage = yield loadImage(templateImageBase64);
    if (templateImage.width < placeholderWidth ||
        templateImage.height < placeholderHeight) {
        throw new Error("Placeholder is bigger than image");
    }
    const templateCanvas = document.createElement("canvas");
    templateCanvas.width = templateImage.width;
    templateCanvas.height = templateImage.height;
    const templateCtx = templateCanvas.getContext("2d");
    templateCtx.drawImage(templateImage, 0, 0, templateImage.width, templateImage.height);
    const templateImgData = templateCtx.getImageData(0, 0, templateCanvas.width, templateCanvas.height);
    const placeholderCoordinates = findChromakeyBoundaries(templateImage.width, templateImage.height, templateImgData, placeholderWidth, placeholderHeight);
    // -2 is for QR to slightly cover borders. To avoid green mask bulging out
    const scaleX = ((qrCodeImage.width - 2) / placeholderWidth) * scale;
    const scaleY = ((qrCodeImage.height - 2) / placeholderHeight) * scale;
    qrCodeImage.width *= scale;
    qrCodeImage.height *= scale;
    const bannerCanvas = document.createElement("canvas");
    const scaledTemplateW = Math.floor(templateImage.width * scaleX);
    const scaledTemplateH = Math.floor(templateImage.height * scaleY);
    bannerCanvas.width = scaledTemplateW;
    bannerCanvas.height = scaledTemplateH;
    const bannerCtx = bannerCanvas.getContext("2d");
    // bannerCtx
    bannerCtx.drawImage(templateImage, 0, 0, templateImage.width, templateImage.height, 0, 0, scaledTemplateW, scaledTemplateH);
    // +1 hides green border
    bannerCtx.drawImage(qrCodeImage, placeholderCoordinates.fromL * scaleX, placeholderCoordinates.fromT * scaleY, qrCodeImage.width, qrCodeImage.height);
    return bannerCanvas.toDataURL();
});
const loadImage = (imageSrc) => {
    return new Promise((resolve, reject) => {
        const templateImg = document.createElement("img");
        templateImg.src = imageSrc;
        templateImg.onload = () => resolve(templateImg);
        templateImg.onerror = () => reject();
    });
};
exports.default = {
    putQrCodeOnChromakeyTemplate,
};
