export interface Participant {
  name: string;
  email: string;
}

export interface Attachment {
  name: string;
  url?: string;
  size: number;
  hash: string;
  hashAlg: string;
  signature: string;
}

export interface MessageData {
  statusId: string;
  sender: Participant;
  subject: string;
  date: string;
  recipients: {
    to: Array<Participant>;
    cc?: Array<Participant>;
  };
  attachments: Array<Attachment>;
  senderPublicKeyUuid: string;
}

export interface StatusData {
  status?: StatusObject;
  statusRaw: string;
  statusClassName?: string;
  merkleTreeId: string;
  transactionHash: string;
}

export interface MerkleTreeVerificationDetails {
  statusPosition: number;
  treeSize: number;
  verified: boolean;
  rootHash: string;
}

export interface MerkleTree {
  rootHash: string;
  nodes: Array<string>;
}

export interface VerificationData {
  statusData?: StatusData;
  blockData?: BlockData;
  txData?: TxData;
  merkleTreeDetails?: MerkleTreeVerificationDetails;
  verificationError?: string;
}

export interface BlockData {
  blockHeight: number;
  timestamp: number;
  confirmationsAmount: number;
}

export interface TxData {
  hash: string;
  blockHeight: number;
  tx: {
    callData: string;
  };
}

export interface TxMerkleTreeData {
  merkleeTreeFileName: string;
  rootNodeHash: string;
}

export interface AttachmentSignature {
  value: string;
  preSignHashAlgorithm: string;
}

export interface SenderStatusObject {
  signatureTime: number;
  signatureTimeSignature: string;
  messageSignature?: string;
  attachmentsSignatures: { [key: string]: { list: AttachmentSignature[] } };
  messageSignatureHashingAlgorithm: string;
  senderSystem: string;
  qrCodeSignature?: string;
  htmlPartsSignature?: string;
  plainPartsSignature?: string;
  pseudoPlainPartsSignature?: string;
  attachmentsPartsSignature?: string;
}

export interface RecipientStatusObjectData {
  creationDate: number;
  gatewayDomainHash: string;
  randomToken: string;
}

export interface RecipientStatusObject {
  iv: string;
  encryptedContent: string;
  decryptors: { [key: string]: string };
}

export interface WrapperData {
  data: string | ArrayBuffer;
  version: number;
  className: string;
}

export interface QrCodeData {
  encryptedKey: Uint8Array;
  encryptedData: Uint8Array;
  nonce: Uint8Array;
}

export type StatusObject = SenderStatusObject | RecipientStatusObject;

// Types
export interface KeyDataPair {
  key: string;
  data: string;
}
